#include "common_header.h"

#include "spotLight.h"

CSpotLight::CSpotLight()
{
	vColor = glm::vec3(1.0f, 1.0f, 1.0f);
	vPosition = glm::vec3(0.0f, 0.0f, 0.0f);
	vDirection = glm::vec3(0.0f, 0.0f, -1.0f);

	bOn = 1;
	fConeAngle = 20.0f;
	fLinearAtt = 0.02f;
}

CSpotLight::CSpotLight(glm::vec3 a_vColor, glm::vec3 a_vPosition, glm::vec3 a_vDirection, int a_bOn, float a_fConeAngle, float a_fLinearAtt)
{
	vColor = a_vColor;
	vPosition = a_vPosition;
	vDirection = a_vDirection;
	
	bOn = a_bOn;
	fConeAngle = a_fConeAngle;
	fLinearAtt = a_fLinearAtt;
}

/*-----------------------------------------------

Name:	SetUniformData

Params:	spProgram - shader program
		sLightVarName - name of spot light variable

Result:	Sets all spot light data.

/*---------------------------------------------*/

void CSpotLight::SetUniformData(CShaderProgram* spProgram, string sLightVarName)
{
	spProgram->SetUniform(sLightVarName+".vColor", vColor);
	spProgram->SetUniform(sLightVarName+".vPosition", vPosition);
	spProgram->SetUniform(sLightVarName+".vDirection", vDirection);

	spProgram->SetUniform(sLightVarName+".bOn", bOn);
	spProgram->SetUniform(sLightVarName+".fConeAngle", fConeAngle);
	spProgram->SetUniform(sLightVarName+".fConeCosine", float(cos(fConeAngle*3.1415f/180.0f)));
	spProgram->SetUniform(sLightVarName+".fLinearAtt", fLinearAtt);
}

